@@ -1,5 +1,4 @@
 import base64
-import io
 import json
 
 import requests
@@ -119,13 +118,22 @@ class APIClient:
 
         return data
 
-    def send_request(self, method, endpoint, params=None, data=None):
+    def send_request(
+        self,
+        method,
+        endpoint,
+        custom_headers={},
+        params=None,
+        data=None,
+        json_data=None,
+    ):
         response = requests.request(
             method=method,
             url=self.api_url(endpoint),
-            headers=self.request_headers(),
+            headers=self.request_headers(custom_headers=custom_headers),
             params=params,
-            json=data,
+            data=data,
+            json=json_data,
         )
         if 400 <= response.status_code < 500:
             raise ClientError(response)
@@ -164,7 +172,7 @@ class APIClient:
         response = self.send_request(
             method="post",
             endpoint="/activate_license",
-            data=data,
+            json_data=data,
         )
         response_json = response.json()
 
@@ -175,6 +183,9 @@ class APIClient:
                 response_json.get("validity_period"),
                 response_json.get("license_signature"),
             )
+            self.signature_verifier.verify_license_signature_v2(
+                response.content, response.headers.get("LicenseSignature")
+            )
 
         return response_json
 
@@ -188,7 +199,7 @@ class APIClient:
         response = self.send_request(
             method="post",
             endpoint="/deactivate_license",
-            data=data,
+            json_data=data,
         )
         return True if response.status_code == 200 else False
 
@@ -210,10 +221,13 @@ class APIClient:
                 response_json.get("validity_period"),
                 response_json.get("license_signature"),
             )
+            self.signature_verifier.verify_license_signature_v2(
+                response.content, response.headers.get("LicenseSignature")
+            )
 
         return response_json
 
-    def activate_offline(
+    def activate_offline_dump(
         self,
         product,
         hardware_id=None,
@@ -249,9 +263,51 @@ class APIClient:
         )
         data["request_id"] = self.hardware_id_provider.get_request_id()
 
-        return io.StringIO(base64.b64encode(json.dumps(data).encode()).decode())
+        return base64.b64encode(json.dumps(data).encode()).decode()
 
-    def deactivate_offline(
+    def activate_offline_load(self, data, hardware_id=None):
+        decoded_data = base64.b64decode(data).decode()
+        response_json = json.loads(decoded_data)
+
+        if self.verify_license_signature:
+            self.signature_verifier.verify_license_signature(
+                hardware_id if hardware_id else self.hardware_id_provider.get_id(),
+                response_json.get("license_key")
+                if response_json.get("license_key")
+                else response_json.get("username"),
+                response_json.get("validity_period"),
+                response_json.get("license_signature"),
+            )
+
+        return response_json
+
+    def activate_offline(self, data):
+        decoded_data = base64.b64decode(data).decode()
+        license_data = json.loads(decoded_data)
+
+        response = self.send_request(
+            method="post",
+            endpoint="/activate_offline",
+            custom_headers={"Content-type": "text/plain"},
+            data=data,
+        )
+        response_json = response.json()
+
+        if self.verify_license_signature:
+            self.signature_verifier.verify_license_signature(
+                license_data.get("hardware_id")
+                if license_data.get("hardware_id")
+                else self.hardware_id_provider.get_id(),
+                license_data.get("license_key")
+                if license_data.get("license_key")
+                else license_data.get("username"),
+                response_json.get("validity_period"),
+                response_json.get("license_signature"),
+            )
+
+        return response_json
+
+    def deactivate_offline_dump(
         self,
         product,
         hardware_id=None,
@@ -293,24 +349,17 @@ class APIClient:
             },
         )
 
-        return io.StringIO(base64.b64encode(json.dumps(data).encode()).decode())
+        return base64.b64encode(json.dumps(data).encode()).decode()
 
-    def activate_offline_load(self, f, hardware_id=None):
-        value = f.getvalue()
-        decoded_value = base64.b64decode(value).decode()
-        response_json = json.loads(decoded_value)
+    def deactivate_offline(self, data):
+        response = self.send_request(
+            method="post",
+            endpoint="/deactivate_offline",
+            custom_headers={"Content-type": "text/plain"},
+            data=data,
+        )
 
-        if self.verify_license_signature:
-            self.signature_verifier.verify_license_signature(
-                hardware_id if hardware_id else self.hardware_id_provider.get_id(),
-                response_json.get("license_key")
-                if response_json.get("license_key")
-                else response_json.get("username"),
-                response_json.get("validity_period"),
-                response_json.get("license_signature"),
-            )
-
-        return response_json
+        return True if response.status_code == 200 else False
 
     def add_consumption(
         self,
@@ -339,7 +388,7 @@ class APIClient:
         response = self.send_request(
             method="post",
             endpoint="/add_consumption",
-            data=data,
+            json_data=data,
         )
 
         return response.json()
@@ -369,7 +418,7 @@ class APIClient:
         response = self.send_request(
             method="post",
             endpoint="/add_feature_consumption",
-            data=data,
+            json_data=data,
         )
 
         return response.json()
@@ -412,7 +461,7 @@ class APIClient:
         response = self.send_request(
             method="get",
             endpoint="/trial_key",
-            data=data,
+            params=data,
         )
 
         return response.json()
@@ -426,7 +475,7 @@ class APIClient:
         response = self.send_request(
             method="get",
             endpoint="/product_details",
-            data=data,
+            params=data,
         )
 
         return response.json()
@@ -448,7 +497,7 @@ class APIClient:
         response = self.send_request(
             method="post",
             endpoint="/track_device_variables",
-            jsdataon=data,
+            json_data=data,
         )
 
         return True if response.status_code == 200 else False
@@ -468,7 +517,7 @@ class APIClient:
         response = self.send_request(
             method="post",
             endpoint="/floating/release",
-            data=data,
+            json_data=data,
         )
 
         return True if response.status_code == 200 else False
@@ -491,7 +540,7 @@ class APIClient:
         response = self.send_request(
             method="post",
             endpoint="/floating/borrow",
-            data=data,
+            json_data=data,
         )
 
         return response.json()
@@ -506,7 +555,7 @@ class APIClient:
         response = self.send_request(
             method="post",
             endpoint="/change_password",
-            data=data,
+            json_data=data,
         )
 
         return True if response.status_code == 200 else False
@@ -533,7 +582,7 @@ class APIClient:
         response = self.send_request(
             method="get",
             endpoint="/versions",
-            data=data,
+            params=data,
         )
 
         return response.json()
